//Asteroids 3d created by Mitchell C. Dodes cis 588. University Of Michigan Dearborn.
//All appropriate files(.x, .bmp, .mp3, .wav) must be in the same folder as this file 
using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Direct3D = Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectSound;
using Microsoft.DirectX.AudioVideoPlayback;

namespace Asteroids
{
	/// <summary>
	/// Summary description for AsteroidGame.
	/// </summary>
	public class AsteroidGame : System.Windows.Forms.Form
	{
		/// <summary>
		/// Required designer variable.
		/// </summary>
		/// 

		

		private System.ComponentModel.Container components = null;

		public AsteroidGame()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();
			this.SetStyle(ControlStyles.AllPaintingInWmPaint | ControlStyles.Opaque, true);//backbuffer??
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			this.Size = new System.Drawing.Size(300,300);
			this.Text = "Form1";
		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		//[STAThread]
		static void Main() 
		{
			Menu myMenu = new Menu();
			Application.Run(myMenu);
			while(myMenu.StartGame == false)
			{
			}
			myMenu.Dispose();
			
			using(AsteroidGame frm = new AsteroidGame())
			{
				//show form and init graphics engin and sound
				frm.Show();
				frm.InitializeSound();
				frm.InitializeGraphics();
				
				Application.Run(frm);
			 }
			
		}
		public void InitializeSound()//initialize sound device
		{
			soundDevice = new Microsoft.DirectX.DirectSound.Device();
			soundDevice.SetCooperativeLevel(this, CooperativeLevel.Normal);
			backgroundMusic1 = new Audio(@"..\..\oldvenus.mp3", false);//intsantiate audio, but not necessarily the song played
		}
		private void InitializeGraphics()
		{
			//set presentation paramaters
			PresentParameters presentParams = new PresentParameters();
			presentParams.SwapEffect = SwapEffect.Discard;
			presentParams.AutoDepthStencilFormat = DepthFormat.D16;//for zbuffer
			presentParams.EnableAutoDepthStencil = true;//for zbuffer

			//store default adapter
			int adapterOrdinal = Manager.Adapters.Default.Adapter;
			CreateFlags flags = CreateFlags.SoftwareVertexProcessing;

			//check to see if we can use pure hardware device
			Microsoft.DirectX.Direct3D.Caps caps = Manager.GetDeviceCaps(adapterOrdinal, DeviceType.Hardware);

			//do we support hardware vertex processing
			if(caps.DeviceCaps.SupportsHardwareTransformAndLight)
			{
				//replace software vertex processing
				flags = CreateFlags.HardwareVertexProcessing;
			}
			
			//do we support true device?
			if(caps.DeviceCaps.SupportsPureDevice)
			{
				flags |=CreateFlags.PureDevice;
			}

			// Start up full screen
			Format current = Manager.Adapters[0].CurrentDisplayMode.Format;

			if (Manager.CheckDeviceType(0, DeviceType.Hardware, current, current, false))
			{
				// Perfect, this is valid
				presentParams.Windowed = false;
				presentParams.BackBufferFormat = current;
				presentParams.BackBufferWidth = ScreenWidth;
				presentParams.BackBufferHeight = ScreenHeight;
			}
			else
			{
				presentParams.Windowed = true;
			}
			//create device
			device = new Microsoft.DirectX.Direct3D.Device(adapterOrdinal, DeviceType.Hardware, this, flags, presentParams);

			//hook device to reset event
			device.DeviceReset += new EventHandler(this.OnDeviceReset);
			
			//call reset event for first time
			this.OnDeviceReset(device, null);
			
			//create fonts
			scoreFont = new Direct3D.Font(device, new System.Drawing.Font("Arial", 12.0f, FontStyle.Bold));
			gameFont = new Direct3D.Font(device, new System.Drawing.Font("Arial", 36.0f, FontStyle.Bold | FontStyle.Italic));
			lifeFont = new Direct3D.Font(device, new System.Drawing.Font("Arial", 12.0f, FontStyle.Bold));
			missionFont = new Direct3D.Font(device, new System.Drawing.Font("Arial", 12.0f, FontStyle.Bold));
			levelFont = new Direct3D.Font(device, new System.Drawing.Font("Arial", 12.0f, FontStyle.Bold));
			
			//load game with default settings
			LoadDefaultGameOptions();
		}
		

		private void OnDeviceReset(object sender, EventArgs e)
		{
			device.RenderState.CullMode = Cull.None;//no backface culling
			device.Transform.Projection = Matrix.PerspectiveFovLH((float)Math.PI / 4,//set up viewport
				(float)this.Width / (float)this.Height, 1.0f, 1000.0f);

			device.Transform.View = Matrix.LookAtLH(new Vector3(0.0f, 1.5f, 17.0f), new Vector3(),//set up viewport
				new Vector3(0, 1, 0));

			//do we have enough support for lights?***********this line could have an error (unit)device.DeviceCaps...
			if((device.DeviceCaps.VertexProcessingCaps.SupportsDirectionalLights) && (device.DeviceCaps.MaxActiveLights > 1))
			{

				//first light
				device.Lights[0].Type = LightType.Directional;
				device.Lights[0].Diffuse = Color.White;
				device.Lights[0].Direction = new Vector3(1, -1, -1);
				device.Lights[0].Update();
				device.Lights[0].Enabled = true;

				//Light two
				device.Lights[1].Type = LightType.Directional;
				device.Lights[1].Diffuse = Color.White;
				device.Lights[1].Direction = new Vector3(-1, 1, -1);
				device.Lights[1].Update();
				device.Lights[1].Enabled = true;
			}
			else
			{
				//no light support, so use ambient
				//ambient light
				device.RenderState.Ambient = Color.White;
			}
			//create star mesh
			starMesh = LoadMesh(device, @"..\..\Stars.x", ref starMaterials, ref starTextures);
			
			//create ship
			ship = new Ship(device);
			
			//try to set up a texture minify filter to minimize pixelation
			if(device.DeviceCaps.TextureFilterCaps.SupportsMinifyAnisotropic)
			{
				device.SamplerState[0].MinFilter = TextureFilter.Anisotropic;
			}
			else if(device.DeviceCaps.TextureFilterCaps.SupportsMinifyLinear)
			{
				device.SamplerState[0].MinFilter = TextureFilter.Linear;
			}

			//do same for magnify filter
			if(device.DeviceCaps.TextureFilterCaps.SupportsMagnifyAnisotropic)
			{
				device.SamplerState[0].MagFilter = TextureFilter.Anisotropic;
			}
			else if(device.DeviceCaps.TextureFilterCaps.SupportsMagnifyLinear)
			{
				device.SamplerState[0].MagFilter = TextureFilter.Linear;
			}

			//instantiate obstacle and cannon
			obstacles = new Obstacles();
			cannons = new Cannons();
			
			
		}

		private void LoadDefaultGameOptions()//default game options, loaded only at start of game
		{
			//deal with music
			if(backgroundMusic1.Playing == false)
			{
				
				int b = Utility.Rnd.Next(1,5);
				if(b == 1)
				{
					backgroundMusic1.Open(@"..\..\oldvenus.mp3", true);
				}
				if(b == 2)
				{
					backgroundMusic1.Open(@"..\..\oldangel.mp3", true);
				}
				if(b == 3)
				{
					backgroundMusic1.Open(@"..\..\flyer.mp3", true);
				}
				if(b == 4)
				{
					backgroundMusic1.Open(@"..\..\bodybag.mp3", true);
				}
				if(b == 5)
				{
					backgroundMusic1.Open(@"..\..\leader.mp3", true);
				}
			}
			//load all obstacle meshes
			obstacleMesh1 = LoadMesh(device, @"..\..\asteroid1.x", ref obstacleMaterials1, ref obstacleTextures1);
			obstacleMesh2 = LoadMesh(device, @"..\..\asteroid2.x", ref obstacleMaterials2, ref obstacleTextures2);
			obstacleMesh3 = LoadMesh(device, @"..\..\asteroid3.x", ref obstacleMaterials3, ref obstacleTextures3);
			obstacleMesh4 = LoadMesh(device, @"..\..\Meteor2.x", ref obstacleMaterials4, ref obstacleTextures4);
			obstacleMesh5 = LoadMesh(device, @"..\..\Saucer.x", ref obstacleMaterials5, ref obstacleTextures5);
			
			// Game space information
			spaceDepth0 = 0.0f;
			spaceDepth1 = -100.0f;
			spaceSpeed = 30.0f;
			spaceSpeedIncrement = 0.5f;
			
			// ship data information
			ship.LocationX = 0.5f;
			ship.LocationY = 0;
			ship.Speed = 10.0f;
			ship.IsMovingLeft = false;
			ship.IsMovingRight = false;
			ship.IsMovingDown = false;
			ship.IsMovingUp = false;
			
			//reset game status data
			score = 0;
			lives = 9;
			objective = 75;
			status = 0;
			level = 1;

			 //Remove any obstacles currently in the game
			foreach(Obstacle o in obstacles)
			{
				// Dispose it first
				o.Dispose();
			}
			foreach(Cannon c in cannons)
			{
				// Dispose it first
				c.Dispose();
			}
			
			obstacles.Clear();
			cannons.Clear();
		
			// Start our timer
			Utility.Timer(DirectXTimer.Start);
		}
		
		private void LoadLevel()//called whenever a level is completed or a player crashes, but game is not over
		{
			
			// game space information
			spaceDepth0 = 0.0f;
			spaceDepth1 = -100.0f;
			spaceSpeed = 50.0f;
			// ship data information
			ship.LocationX = 0.5f;
			ship.LocationY = 0;
			ship.Speed = 10.0f;
			ship.IsMovingLeft = false;
			ship.IsMovingRight = false;
			ship.IsMovingDown = false;
			ship.IsMovingUp = false;
			
			//these if statments set parameters that change between levels
			if(level == 2)
			{
				objective = 100;
				spaceSpeedIncrement = 1.0f;
			}
			if(level == 3)
			{
				objective = 125;
				spaceSpeedIncrement = 2.0f;
			}
			if(level == 4)
			{
				objective = 135;
				spaceSpeedIncrement = 3.0f;
			}
			if(level == 5)
			{
				objective = 150;
				spaceSpeedIncrement = 4.0f;
			}
			if(level == 6)
			{
				//game is over
			}

			

			//Remove any obstacles currently in the game
			foreach(Obstacle o in obstacles)
			{
				// Dispose it first
				o.Dispose();
			}
			foreach(Cannon c in cannons)
			{
				// Dispose it first
				c.Dispose();
			}
			
			obstacles.Clear();
			cannons.Clear();
		
			// Start our timer
			Utility.Timer(DirectXTimer.Start);
		}
		private void DrawStars(float x, float y, float z)//draws the star mesh environment
		{
			//translate mesh to correct position and render each subset
			device.Transform.World = Matrix.Translation(x, y, z);
			for(int i = 0; i < starMaterials.Length; i++)
			{
				device.Material = starMaterials[i];
				device.SetTexture(0, starTextures[i]);
				starMesh.DrawSubset(i);
			}
		}
	

		protected override void OnPaint(System.Windows.Forms.PaintEventArgs e)
		{
			//before render, update state
			OnFrameUpdate();
			//clear to black screen
			device.Clear(ClearFlags.Target | ClearFlags.ZBuffer, Color.Black, 1.0f, 0);
			
			//begin scene
			device.BeginScene();
			
			DrawStars(0.0f, -7.0f, spaceDepth0 -50);
			DrawStars(0.0f, -7.0f, spaceDepth1 -50);
			ship.Draw(device);
			foreach(Obstacle o in obstacles)
			{
				o.Draw(device);
			}
			foreach(Cannon c in cannons)
			{
				c.Draw(device);
			}

			//scoring
			if(hasGameStarted)
			{
				
				//draw score
				scoreFont.DrawText(null, string.Format("Current score: {0}", score), new Rectangle(5,5,0,0), DrawTextFormat.NoClip, Color.Yellow);
				lifeFont.DrawText(null, string.Format("Lives X {0}", lives), new Rectangle(5,20,0,0), DrawTextFormat.NoClip, Color.Turquoise);
				missionFont.DrawText(null, string.Format("Objects Destroyed {0} / {1}", status, objective), new Rectangle(5,35,0,0), DrawTextFormat.NoClip, Color.Turquoise);
				levelFont.DrawText(null, string.Format("Level {0}", level), new Rectangle(5,50,0,0), DrawTextFormat.NoClip, Color.Turquoise);
				
			}
			if(isGameOver)
			{
				//notify player that game has ended
				if(hasGameStarted)
				{
					UserHasControl = false;//this insures that a player cannot press buttons while the system is providing info
					gameFont.DrawText(null, "You've failed. The game is over.", new Rectangle(25, 75, 0, 0), DrawTextFormat.NoClip, Color.Red);
				}
				if((System.Environment.TickCount - gameOverTick) >= 1000)
				{
					//only draw this when game has been over more than a second
					gameFont.DrawText(null, "Press any key to start.", new Rectangle(25, 130, 0, 0), DrawTextFormat.NoClip, Color.WhiteSmoke);
					UserHasControl = true;
				}
			}
			if(hasCrashed && !isGameOver)//when a crash occurs but game is not over
			{
				//notify player that game has ended
				if(hasGameStarted)
				{
					UserHasControl = false;
					gameFont.DrawText(null, "You've Crashed.", new Rectangle(25, 75, 0, 0), DrawTextFormat.NoClip, Color.Red);
				}
				if((System.Environment.TickCount - gameOverTick) >= 1000)
				{
					//only draw this when game has been over more than a second
					gameFont.DrawText(null, "Press any key to continue.", new Rectangle(25, 130, 0, 0), DrawTextFormat.NoClip, Color.WhiteSmoke);
					UserHasControl = true;
				}
			}
			if(hasCompletedLevel && !hasCompletedGame)//when a level is completed but game is not
			{
				//notify player that game has ended
				if(hasGameStarted)
				{
					gameFont.DrawText(null, "Level Complete. Great Job!", new Rectangle(25, 75, 0, 0), DrawTextFormat.NoClip, Color.Red);
					UserHasControl = false;
				}
				if((System.Environment.TickCount - gameOverTick) >= 1000)
				{
					//only draw this when game has been over more than a second
					gameFont.DrawText(null, "Press any key to start next level.", new Rectangle(25, 130, 0, 0), DrawTextFormat.NoClip, Color.WhiteSmoke);
					UserHasControl = true;
				}
			}
			if(hasCompletedGame)//when game is completed
			{
				//notify player that game has ended
				if(hasGameStarted)
				{
					gameFont.DrawText(null, "Mission Complete. Great Job.", new Rectangle(25, 75, 0, 0), DrawTextFormat.NoClip, Color.Red);
					UserHasControl = false;
				}
				if((System.Environment.TickCount - gameOverTick) >= 1000)
				{
					//only draw this when game has been over more than a second
					gameFont.DrawText(null, "Press escape key end game.", new Rectangle(25, 130, 0, 0), DrawTextFormat.NoClip, Color.WhiteSmoke);
					
				}
			}
		
			//end scene
			device.EndScene();
			
			//present
			device.Present();
			
			//force paint
			this.Invalidate();
		}

		private void OnFrameUpdate()//incrementel update of the system
		{
			//nothing to update if game is over
			if((isGameOver) || (!hasGameStarted))
			{
				return;
			}
			//if music has ended, pick a new song at random
			if(backgroundMusic1.CurrentPosition == backgroundMusic1.Duration)
			{
				backgroundMusic1.Stop();
				int b = Utility.Rnd.Next(1,5);
				if(b == 1)
				{
					backgroundMusic1.Open(@"..\..\oldvenus.mp3", true);
				}
				if(b == 2)
				{
					backgroundMusic1.Open(@"..\..\oldangel.mp3", true);
				}
				if(b == 3)
				{
					backgroundMusic1.Open(@"..\..\flyer.mp3", true);
				}
				if(b == 4)
				{
					backgroundMusic1.Open(@"..\..\bodybag.mp3", true);
				}
				if(b == 5)
				{
					backgroundMusic1.Open(@"..\..\leader.mp3", true);
				}
			}

			//get elapsed time
			elapsedTime = Utility.Timer(DirectXTimer.GetElapsedTime);

			//move space toward player
			spaceDepth0 += (spaceSpeed * elapsedTime);
			spaceDepth1 += (spaceSpeed * elapsedTime);

			//check if road needs to be cycled
			if(spaceDepth0 > 75.0f)//if space depth 0 is 75% of the way to the player cycle 
			{
				spaceDepth0 = spaceDepth1 - 100.0f;
				AddObstacles(spaceDepth0 - 100);
			}

			if(spaceDepth1 > 75.0f)// if space depthe 1  is 75% of the way to the player cycle
			{
				spaceDepth1 = spaceDepth0 - 100.0f;
				AddObstacles(spaceDepth1 - 100);
			}
			ship.Update(elapsedTime);//update ship
			foreach(Cannon c in cannons)//update each cannon that has been fired
			{
				c.Update( elapsedTime, spaceSpeed);
			}
			
			Cannons removeCannons = new Cannons();//arraylist to hold cannons that are outsid of play bounds
			foreach(Cannon c in cannons)//for all cannons
			{
				//max distance of screen 
				if(c.Depth < -SpaceSize)
				{
					removeCannons.Add(c);//add cannon to remove cannon array
				}
			
			}
			
			//remove any obstacles that are past the ship and update score for each one
			//increase speed by .5 of the speed increment to make game harder
			Obstacles removeObstacles = new Obstacles();
			foreach(Obstacle o in obstacles)
			{
				if(o.Depth > ship.Diameter - (Ship.Depth * 2))
				{
					//add this pbstacle to list to be removed
					removeObstacles.Add(o);
					spaceSpeed += spaceSpeedIncrement/2;
					//make sure road speed stays below max
					if(spaceSpeed >= MaximumspaceSpeed)
					{
						spaceSpeed = MaximumspaceSpeed;
					}

					//increase ship speed
					ship.IncrementSpeed();
					
				}
			}
			
			//move obstacles
			foreach(Obstacle o in obstacles)
			{
				foreach(Cannon c in cannons)//check if any cannon has hit an obstacle and remove cannon and obstacle
					//also increment scor and speed and check if level objective has been met
				{
					if(c.IsHittingObstacle(o.LocationX, o.LocationY, o.LocationZ, o.Radius))
					{
						//add new score
						score += (int)(spaceSpeed * (spaceSpeed / ship.Speed));
						//increment speed when a obstacle is hit
						spaceSpeed += spaceSpeedIncrement;
						status++;
						//check for completion of level
						if(status == objective)
						{
							hasCompletedLevel = true;
							gameOverTick = System.Environment.TickCount;

							//stop timer
							Utility.Timer(DirectXTimer.Stop);
							status = 0;
							//increment level
							level++;
							//load next level
							if(level == 6)
							{
								hasCompletedGame = true;
							}
						}
						//make sure road speed stays below max
						if(spaceSpeed >= MaximumspaceSpeed)
						{
							spaceSpeed = MaximumspaceSpeed;
						}

						//increase ship speed
						ship.IncrementSpeed();
						
						//remove obstacle
						o.PlayExplosion(soundDevice);
						removeObstacles.Add(o);
						
						//remove cannon
						removeCannons.Add(c);
					}
				}
				//update obstacle check to see if it hits ship
				o.Update(elapsedTime, spaceSpeed);
				if(o.IsHittingShip(ship.LocationX, ship.LocationY, ship.LocationZ, ship.Diameter))
				{
					//if ship is hit, game over
					if(lives > 0)
					{
						o.PlayExplosion(soundDevice);
						hasCrashed = true;
						gameOverTick = System.Environment.TickCount;
						
						//stop timer
						Utility.Timer(DirectXTimer.Stop);
						lives--;
					}
					else//ship is hit but not game over
					{
						o.PlayExplosion(soundDevice);
						isGameOver = true;
						gameOverTick = System.Environment.TickCount;

						//stop timer
						Utility.Timer(DirectXTimer.Stop);
					}
					
					removeObstacles.Add(o);// ad all obstacles that are out of play to array
				}
			}
			foreach(Cannon c in removeCannons)//clear all cannons that are in the remove cannon array
			{
				cannons.Remove(c);
				//dispose
				c.Dispose();
			}
			removeCannons.Clear();//clear array
			//remove obstacles from list
			foreach(Obstacle o in removeObstacles)
			{
				obstacles.Remove(o);
				//dispose
				o.Dispose();
			}
			removeObstacles.Clear();
			

		}
		


		/// <summary>
		///generic mesh loading routine
		/// </summary>
		public static Mesh LoadMesh(Microsoft.DirectX.Direct3D.Device device, string file, ref Material[] meshMaterials, ref Texture[] meshTextures)
		{
			ExtendedMaterial[] mtrl;//stores info about subsets of mesh

			//load mesh
			Mesh tempMesh = Mesh.FromFile(file, MeshFlags.Managed, device, out mtrl);

			//if there are materials, store them
			if((mtrl != null) && (mtrl.Length > 0))
			{
				meshMaterials = new Material[mtrl.Length];
				meshTextures = new Texture[mtrl.Length];

				//store each material and texture
				for(int i = 0; i< mtrl.Length; i++)
				{
					meshMaterials[i] = mtrl[i].Material3D;
					if((mtrl[i].TextureFilename != null) && (mtrl[i].TextureFilename != string.Empty))
					{
						//there is a texture, load it
						meshTextures[i] = TextureLoader.FromFile(device, @"..\..\" + mtrl[i].TextureFilename);
					}
				}
			}
			return tempMesh;
		}
		protected override void OnKeyUp(System.Windows.Forms.KeyEventArgs e)//deal with release of keys
		{
			if((e.KeyCode == Keys.Left) || (e.KeyCode == Keys.NumPad4))
			{
				ship.IsNotMoving = true;
			}
			//handle move right

			if((e.KeyCode == Keys.Right) || (e.KeyCode == Keys.NumPad6))
			{
				ship.IsNotMoving = true;
			}
			if((e.KeyCode == Keys.NumPad2) || (e.KeyCode == Keys.Up))
			{
				ship.IsNotMoving = true;
			}
			if((e.KeyCode == Keys.NumPad8) || (e.KeyCode == Keys.Down))
			{
				ship.IsNotMoving = true;
			}
			if((e.KeyCode == Keys.Space))
			{
				
				hasFired = false;
			}
		}
		

		protected override void OnKeyDown(System.Windows.Forms.KeyEventArgs e)//deal with press of keys
		{
			
			//handle quit
			if(e.KeyCode == Keys.Escape)
			{
				if(hasCompletedGame)//limits user control during parts of the game
				{
					this.Hide();
					Menu myMenu = new Menu();
					myMenu.ChangePic();
					myMenu.Show();
					System.Threading.Thread.Sleep(10000);
				}
				this.Close();
				return;
			}
			if(UserHasControl)
			{
				//handle move left

				if((e.KeyCode == Keys.Left) || (e.KeyCode == Keys.NumPad4))
				{
					ship.IsMovingLeft = true;
					ship.IsMovingRight = false;
					ship.IsMovingUp = false;
					ship.IsMovingDown = false;
					ship.IsNotMoving = false;
				}
				//handle move right

				if((e.KeyCode == Keys.Right) || (e.KeyCode == Keys.NumPad6))
				{
					ship.IsMovingLeft = false;
					ship.IsMovingRight = true;
					ship.IsMovingUp = false;
					ship.IsMovingDown = false;
					ship.IsNotMoving = false;
				}
				if((e.KeyCode == Keys.NumPad5) || (e.KeyCode == Keys.S))
				{
					ship.IsMovingRight = false;
					ship.IsMovingLeft = false;
					ship.IsMovingUp = false;
					ship.IsMovingDown = false;
				
				}
				if((e.KeyCode == Keys.NumPad2) || (e.KeyCode == Keys.Up))
				{
					ship.IsMovingUp = true;
					ship.IsMovingDown = false;
					ship.IsMovingRight = false;
					ship.IsMovingLeft = false;
					ship.IsNotMoving = false;
				}
				if((e.KeyCode == Keys.NumPad8) || (e.KeyCode == Keys.Down))
				{
					ship.IsMovingUp = false;
					ship.IsMovingDown = true;
					ship.IsMovingRight = false;
					ship.IsMovingLeft = false;
					ship.IsNotMoving = false;
				}
				if((e.KeyCode == Keys.Space))
				{
					if(hasFired == false)
					{
						Cannon c = new Cannon(device, ship.LocationX + 0.5f, ship.LocationY, ship.LocationZ -1);
						c.playCannon(soundDevice);
						//fire cannon the extra numbers are to fine tune the location that the cannon appears to fire from
						cannons.Add(c);
						cannons.Add(new Cannon(device, ship.LocationX - 1.3f, ship.LocationY, ship.LocationZ -1));
						hasFired = true;
					}
				}
				if(isGameOver)
				{
					LoadDefaultGameOptions();//restart game
				}
				if(hasCompletedLevel || hasCrashed )
				{
					LoadLevel();
				}
			

				//set isGameOver to false when a key is pressed
				isGameOver = false;
				hasGameStarted = true;
				hasCrashed = false;
				hasCompletedLevel = false;

				//ignore keystrokes for a second after game is over
				if((System.Environment.TickCount - gameOverTick) < 1000)
				{
					return;
				}
			}
			
		}

		/// <summary>
		/// add a series of obstacles to the road
		/// </summary>
		/// <paramref name=" minDepth">Minimum depth of obstacles</paramref>
		private void AddObstacles(float minDepth)
		{
			//add right number of obstacles
			int numberToAdd = (int)((SpaceSize / ship.Diameter - 1) / 4.0f);
			//get the minimum space between obstacles in this section
			float minSize = ((SpaceSize / numberToAdd) - ship.Diameter) /2.0f;
			
			for(int i = 0; i < numberToAdd; i++)
			{
				//get random number in minimum size range
				float depth = minDepth - ((float)Utility.Rnd.NextDouble() * minSize);
				//makse sure in range
				depth -= (i * (ship.Diameter * 2));

				//pick position between -5<x<5 and -5<y<5
				float locationX = (Utility.Rnd.Next(-5, 5));
				float locationY = (Utility.Rnd.Next(-5, 5));
				//add obstacle
				if(level == 1)//pick a random mesh to add as an obstacle
				{
					int a = Utility.Rnd.Next(1,4);
					if(a == 1)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh1, obstacleMaterials1, obstacleTextures1, true));
				
					}
					if(a == 2)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh2, obstacleMaterials2, obstacleTextures2, true));
				
					}
					if(a == 3)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh3, obstacleMaterials3, obstacleTextures3, true));
				
					}
				}
				if(level == 2)//pick a specific mesh to add as an obstacle
				{
					obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh4, obstacleMaterials4, obstacleTextures4, true));
					
				}
				if(level == 3)//pick a random mesh to add as an obstacle
				{
					int a = Utility.Rnd.Next(1,5);
					if(a == 1)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh1, obstacleMaterials1, obstacleTextures1, true));
				
					}
					if(a == 2)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh2, obstacleMaterials2, obstacleTextures2, true));
				
					}
					if(a == 3)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh3, obstacleMaterials3, obstacleTextures3, true));
				
					}
					if(a == 4)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh4, obstacleMaterials4, obstacleTextures4, true));
					
					}
				}
				if(level == 4)//pick a specific mesh to add as an obstacle
				{
					obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh5, obstacleMaterials5, obstacleTextures5, false));
				
				}
				if(level == 5)//pick a random mesh to add as an obstacle
				{
					int a = Utility.Rnd.Next(1,6);
					if(a == 1)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh1, obstacleMaterials1, obstacleTextures1, true));
				
					}
					if(a == 2)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh2, obstacleMaterials2, obstacleTextures2, true));
				
					}
					if(a == 3)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh3, obstacleMaterials3, obstacleTextures3, true));
				
					}
					if(a == 4)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh4, obstacleMaterials4, obstacleTextures4, true));
					
					}
					if(a == 5)
					{
						obstacles.Add(new Obstacle(device, locationX, locationY, depth , obstacleMesh5, obstacleMaterials5, obstacleTextures5, false));
					}

				}
			}
		}


		private Microsoft.DirectX.Direct3D.Device device = null;

		//variables to hold game boar mesh data
		private Mesh starMesh = null;
		private Material[] starMaterials = null;
		private Texture[] starTextures = null;
		


		//constant values for the locations
		public const float Left = 5.0f;
		public const float Right = -3.8f;//this is because the ship is not aligned at the origin
		public const float Top = 5.0f;
		public const float Bottom = -5.0f;
		private const float SpaceSize = 100.0f;
		private const float MaximumspaceSpeed = 250.0f;

		//speed increment for when an obstacle is hit
		private float spaceSpeedIncrement = 0.5f;


		//depth locations of the 2 star meshes
		private float spaceDepth0 = 0.0f;
		private float spaceDepth1 = -100.0f;
		private float spaceSpeed = 30.0f;

		private float elapsedTime = 0.0f;
		//ship
		private Ship ship = null;

		//obstacle
		private Obstacles obstacles;
		private Cannons cannons;
		private const float ObstacleHeight = Ship.Height;

		//game state info
		private bool UserHasControl = false;
		private bool hasCrashed = false;
		private bool hasCompletedLevel = false;
		private bool isGameOver = true;
		private bool hasCompletedGame = false;
		private int gameOverTick = 0;
		private int score = 0;
		private int lives = 8;
		private bool hasGameStarted = false;

		//font info
		private Direct3D.Font scoreFont = null;
		private Direct3D.Font gameFont = null;
		private Direct3D.Font missionFont = null;
		private Direct3D.Font lifeFont = null;
		private Direct3D.Font levelFont = null;

		//sound
		private Microsoft.DirectX.DirectSound.Device soundDevice = null;
		
		//for fire
		private bool hasFired = false;
		
		//for music playback
		private Audio backgroundMusic1;
		
		//screen info
		public const int ScreenWidth = 800;
		public const int ScreenHeight = 600;
		
		

		//obstacle meshes
		private Mesh obstacleMesh1 = null;
		private Material[] obstacleMaterials1 = null;
		private Texture[] obstacleTextures1 = null;

		private Mesh obstacleMesh2 = null;
		private Material[] obstacleMaterials2 = null;
		private Texture[] obstacleTextures2 = null;
		
		private Mesh obstacleMesh3 = null;
		private Material[] obstacleMaterials3 = null;
		private Texture[] obstacleTextures3 = null;

		private Mesh obstacleMesh4 = null;
		private Material[] obstacleMaterials4 = null;
		private Texture[] obstacleTextures4 = null;

		private Mesh obstacleMesh5 = null;
		private Material[] obstacleMaterials5 = null;
		private Texture[] obstacleTextures5 = null;

		//level info
		private int level = 1;
		private int objective = 75;
		private int status;




		
	}
	
}
