using System;
using System.Drawing;
using System.Collections;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
using Microsoft.DirectX.DirectSound;

namespace Asteroids
{
	/// <summary>
	/// Summary description for Cannon.
	/// </summary>
	public class Cannon : IDisposable
	{
		public Cannon(Microsoft.DirectX.Direct3D.Device device, float x, float y, float z)
		{
			//store position
			position = new Vector3(x, y, z);
			//cannons are spheres
			cannonMesh = Mesh.Sphere(device, ObjectRadius, ObjectStacksSlices, ObjectStacksSlices);
			
			obstacleMaterial = new Material();
			Color objColor = CannonColors[Utility.Rnd.Next(CannonColors.Length)];
			obstacleMaterial.Ambient = objColor;
			obstacleMaterial.Diffuse = objColor;


			//make rotation axis and speed random
			rotationspeed = (float)Utility.Rnd.NextDouble() * (float)Math.PI;
			rotationVector = new Vector3((float)Utility.Rnd.NextDouble(), (float)Utility.Rnd.NextDouble(), (float)Utility.Rnd.NextDouble());

			
		}
		//updat obstacles position to match the moving road
		public void Update(float elapsedTime, float speed)
		{
			position.Z -= (speed * elapsedTime);
			
			
		}
		public void Draw(Microsoft.DirectX.Direct3D.Device device)
		{
			
			device.Transform.World = Matrix.RotationAxis(rotationVector, rotation) * Matrix.Translation(position); 
			
			device.Material = obstacleMaterial;
			device.SetTexture(0, null);//no texture
			cannonMesh.DrawSubset(0);//draw mesh
			
		}
		//collision detection scheme
		public bool IsHittingObstacle(float obstacleLocationX, float obstacleLocationY, float obstacleLocationZ, float obstacleRadius)
		{
			float obstacleXMin = (obstacleLocationX - obstacleRadius );
			float obstacleXMax = (obstacleLocationX + obstacleRadius );
			float obstacleYMin = (obstacleLocationY - obstacleRadius ); // this kind of forces the bounding sphere into an ellipsoid
			float obstacleYMax = (obstacleLocationY + obstacleRadius ); //because ship is much shorter than its largest dimension
			float obstacleZMin = (obstacleLocationZ - obstacleRadius );
			float obstacleZMax = (obstacleLocationZ + obstacleRadius );
			
			float cannonXMin = position.X - ObjectRadius;
			float cannonXMax = position.X + ObjectRadius;
			float cannonYMin = position.Y - ObjectRadius;
			float cannonYMax = position.Y + ObjectRadius;
			float cannonZMin = position.Z - ObjectRadius;
			float cannonZMax = position.Z + ObjectRadius;


			if(cannonZMin < obstacleZMax + 4 && cannonZMin > obstacleZMin)//a collision is possible adding 4 makes it look better if a wing collides
			{
				//collision from above
				if((cannonYMin < obstacleYMax) && (cannonYMin > obstacleYMin))
				{
					//collision from left
					if((cannonXMin < obstacleXMax) && (cannonXMin > obstacleXMin))
					{
						return true;
					}
					//collision from right
					if((cannonXMax > obstacleXMin) && (cannonXMax < obstacleXMax))
					{
						return true;
					}
				}
				//collision from bottom
				if((cannonYMax > obstacleYMin) && (cannonYMax < obstacleYMax))
				{
					//collision from left
					if((cannonXMin < obstacleXMax) && (cannonXMin > obstacleXMin))
					{
						return true;
					}
					//collision from right
					if((cannonXMax > obstacleXMin) && (cannonXMax < obstacleXMax))
					{
						return true;
					}
				}
				//head on in y axis
				if((cannonYMax > obstacleYMax) && (cannonYMin < obstacleYMin))
				{
					//collision from left
					if((cannonXMin < obstacleXMax) && (cannonXMin > obstacleXMin))
					{
						return true;
					}
					//collision from right
					if((cannonXMax > obstacleXMin) && (cannonXMax < obstacleXMax))
					{
						return true;
					}
				}
			}
			

			return false;
		}
		//play the cannons sound fx
		public void playCannon(Microsoft.DirectX.DirectSound.Device soundDevice)
		{
			torpedoSound = new SecondaryBuffer(@"..\..\Torpedo.wav", soundDevice);
			torpedoSound.Volume = -5;

			torpedoSound.Play(0, BufferPlayFlags.Default);
		}
		public float Depth
		{
			get { return position.Z; }
		}

		#region IDisposable Members

		public void Dispose()
		{
			// TODO:  Add Obstacle.Dispose implementation
		}

		#endregion
		private const int NumberMeshTypes = 5;//sphere, cube, torus, cylinder, teapot
		private const float ObjectLength = 0.5f;
		private const float ObjectRadius = ObjectLength / 2.0f;
		private const int ObjectStacksSlices = 18;

		//obstacle colors
		private static readonly Color[] CannonColors = {
															 Color.BlueViolet};

		//to control rotation
		private float rotation = 0.0f;
		private float rotationspeed = 0.0f;
		private Vector3 rotationVector;
		private Vector3 position;
		//mesh info
		private Mesh cannonMesh = null;
		private Material obstacleMaterial;

		//for sound fx
		private SecondaryBuffer torpedoSound = null;
	
		
	}
	public class Cannons : IEnumerable //to add and remove Cannon from game
	{
		/// <summary>
		/// Indexer for this class for direct access to ann obstacle
		/// </summary>
		
		public Cannon this[int index]
		{
			get
			{
				return (Cannon)cannonList[index];
			}
		}

		//get enumerator from array list
		public IEnumerator GetEnumerator()//so we can use foreach
		{
			return cannonList.GetEnumerator();
		}

		/// <summary>
		/// add obstacle to list
		/// 
		/// </summary>
		/// <paramref name=" obstacle"> the obstacle to add</paramref>
		public void Add(Cannon cannon)
		{
			cannonList.Add(cannon);
		}

		/// <summary>
		/// remove obstacle from list
		/// </summary>
		/// <paramref name=" obstacle">the obstacle to remove</paramref>
		public void Remove(Cannon cannon)
		{
			cannonList.Remove(cannon);
		}

		/// <summary>
		/// clear obstacle list
		/// </summary>
		public void Clear()
		{
			cannonList.Clear();
		}
	
		private ArrayList cannonList = new ArrayList();
	}
}
