using System;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;
namespace Asteroids
{
	/// <summary>
	/// Summary description for Ship.
	/// </summary>
	public class Ship
	{
		/// <summary>
		/// create a Ship device then load mesh data for it
		/// </summary>
		public Ship(Device device)
		{
			//load a mesh to represent the ship
			shipMesh = AsteroidGame.LoadMesh(device, @"..\..\Fighter3.x", ref shipMaterials, ref shipTextures);
			
			//calculate bounding sphere for Ship
			VertexBuffer vb = shipMesh.VertexBuffer;
			try
			{
				//look at entire buffer to calculate bounding sphere
				GraphicsStream stm = vb.Lock(0, 0, LockFlags.None);
				Vector3 center;
				radius = Geometry.ComputeBoundingSphere(stm, shipMesh.NumberVertices, shipMesh.VertexFormat, out center);
				shipDiameter = (radius * 2) * Scale;
				

			}
			finally
			{
				//no matter what unlock and dispose of this vertex buffer
				vb.Unlock();
				vb.Dispose();
			}
		}

		/// <summary>
		/// render Ship with current properties
		/// </summary>
		public void Draw(Device device)
		{
			//scale Ship 
			device.Transform.World = Matrix.Scaling(Scale, Scale, Scale) * Matrix.Translation(shipLocationX, shipLocationY, Depth) * Matrix.RotationZ(roll) * Matrix.RotationX(pitch);
			
			for(int i = 0; i < shipMaterials.Length; i++)//draw ship and materials and textures
			{
				device.Material = shipMaterials[i];
				device.SetTexture(0, shipTextures[i]);
				shipMesh.DrawSubset(i);
			}
		}

		
		/// <summary>
		/// Increment the speed of the Ship
		/// </summary>
		public void IncrementSpeed()
		{
			shipSpeed += SpeedIncrement;
		}

		/// <summary>
		/// update cars state based on elapsed time
		/// </summary>
		/// <param name="elapsedTime">Amount of time that has elapsed</param>
		public void Update(float elapsedTime)
		{
			
			if(movingLeft)
			{
				//roll a bit
				while(roll > -(float)Math.PI/64)
				{
					roll -= 0.001f;
				}
				//move Ship
				shipLocationX += (shipSpeed * elapsedTime);
				//is Ship all the way left?
				if(shipLocationX >= AsteroidGame.Left)
				{
					movingLeft = false;
					shipLocationX = AsteroidGame.Left;
				}
				
			}
			if(movingRight)
			{
				//roll
				while(roll < (float)Math.PI/64)
				{
					roll += 0.001f;
				}
				//move Ship
				shipLocationX -= (shipSpeed * elapsedTime);
				//is Ship all the way right?
				if(shipLocationX <= AsteroidGame.Right)
				{
					movingRight = false;
					shipLocationX = AsteroidGame.Right;
				}
				
			}
			if(movingUp)
			{
				//pitch ship 
				while(pitch < (float)Math.PI/64)
				{
					pitch += 0.001f;
				}
				//move Ship
				shipLocationY += (shipSpeed * elapsedTime);
				//is Ship all the way right?
				if(shipLocationY >= AsteroidGame.Top)
				{
					movingUp = false;
					shipLocationY = AsteroidGame.Top;
				}
			}
			if(movingDown)
			{
				//pitch ship 
				while(pitch > -(float)Math.PI/64)
				{
					pitch -= 0.001f;
				}
				//move Ship
				shipLocationY -= (shipSpeed * elapsedTime);
				//is Ship all the way right?
				if(shipLocationY <= AsteroidGame.Bottom)
				{
					movingDown = false;
					shipLocationY = AsteroidGame.Bottom;
				}
			}
			if(notMoving)
			{
				//level ship
				while(roll < 0.0f)
				{
					roll += 0.001f;
				}
				while(roll > 0.0f)
				{
					roll -= 0.001f;
				}
				
				while(pitch < 0.0f)
				{
					pitch += 0.001f;
				}
				while(pitch > 0.0f)
				{
					pitch -= 0.001f;
				}
				
				movingLeft = false;
				movingRight = false;
				movingUp = false;
				movingDown = false;
			}
			
			

			
		}
		//Public properties for Ship data
		public float LocationX
		{
			get{ return shipLocationX; }
			set{shipLocationX = value; }
		}
		public float LocationY
		{
			get{ return shipLocationY; }
			set{shipLocationY = value; }
		}
		public float LocationZ
		{
			get{ return Depth; }
		}
		public float Diameter
		{
			get{ return shipDiameter;}
		}
		public float Speed
		{
			get{ return shipSpeed; }
			set{ shipSpeed = value; }
		}
		public bool IsMovingLeft
		{
			get{ return movingLeft; }
			set{ movingLeft = value; }
		}
		public bool IsMovingRight
		{
			get{ return movingRight; }
			set{ movingRight = value; }
		}
		public bool IsMovingUp
		{
			get{ return movingUp; }
			set{ movingUp = value; }
		}
		public bool IsMovingDown
		{
			get{ return movingDown; }
			set{ movingDown = value; }
		}
		public bool IsNotMoving
		{
			get{ return notMoving; }
			set{ notMoving = value; }
		}
		

		


		//constants
		public const float Height = -1.5f;
		public const float Depth = 2.0f;
		public const float SpeedIncrement = 0.1f;
		private const float Scale = 0.25f;

		//Ship data info
		private float shipLocationX = AsteroidGame.Right;
		public float shipLocationY = AsteroidGame.Top;
		private float shipDiameter;
		private float radius;
		private float shipSpeed = 10.0f;
		private bool movingLeft = false;
		private bool movingRight = false;
		private bool movingUp = false;
		private bool movingDown = false;
		private bool notMoving = false;
		//Ship mesh info
		private Mesh shipMesh = null;
		private Material[] shipMaterials;
		private Texture[] shipTextures;

		

		//to handle roll and pitch of ship when it moves
		private float roll = 0.0f;
		private float pitch = 0.0f;
		
	}
}
